@@ -3,23 +3,23 @@ mod date_time_picker;
 mod error;
 mod frequency_label;
 mod indicator_icons;
+mod intermediate_location_item;
 mod journey_detail_page;
 mod journey_list_item;
 mod journey_store_item;
 mod journeys_page;
 mod leg_item;
 mod place_list_item;
-mod preferences_window;
+mod preferences_dialog;
 mod provider_list_item;
 mod provider_popover;
 mod refresh_button;
 mod remark_item;
 mod search_options_button;
-mod search_options_window;
+mod search_options_dialog;
 mod search_page;
 mod search_store_item;
 mod station_entry;
-mod stopover_item;
 mod stores;
 mod time_divider;
 mod transition;
