@@ -52,11 +52,11 @@ pub mod imp {
         INVALID_LIST_POSITION,
     };
     use gtk::{CompositeTemplate, Popover};
-    use hafas_rs::api::locations::LocationsOptions;
     use libadwaita::subclass::prelude::{EntryRowImpl, PreferencesRowImpl};
     use once_cell::sync::Lazy;
+    use rcore::LocationsOptions;
 
-    use crate::backend::{HafasClient, Place, RequestLimiter};
+    use crate::backend::{Client, Place, RequestLimiter};
     use crate::gui::place_list_item::PlaceListItem;
 
     const REQUEST_DURATION: Duration = Duration::from_secs(1);
@@ -74,7 +74,7 @@ pub mod imp {
         pub(super) completions: RefCell<ListStore>,
         selection: RefCell<SingleSelection>,
 
-        client: RefCell<Option<HafasClient>>,
+        client: RefCell<Option<Client>>,
 
         place: RefCell<Option<Place>>,
 
@@ -131,6 +131,8 @@ pub mod imp {
         // Adapted from <https://gitlab.gnome.org/GNOME/epiphany/-/blob/b6203597637b4b7725372750983f6e38ca92c2ac/src/ephy-location-entry.c#L436-512>.
         #[template_callback]
         fn handle_key_pressed(&self, key: Key, _: u32, modifier: ModifierType) -> Propagation {
+            let obj = self.obj();
+
             if modifier.intersects(
                 gdk::ModifierType::SHIFT_MASK
                     .union(gdk::ModifierType::ALT_MASK)
@@ -178,6 +180,21 @@ pub mod imp {
                 }
             };
 
+            if let Some(new_selected_item_name) = new_selected
+                .and_then(|i| model.item(i))
+                .and_downcast_ref::<Place>()
+                .and_then(|p| p.name())
+            {
+                // Translators: Text that will be announced by the screen reader when a suggestion was selected.
+                let format = gettextrs::gettext("Suggestion {} selected")
+                    .replace("{}", &new_selected_item_name);
+                obj.announce(&format, gtk::AccessibleAnnouncementPriority::Low);
+            } else {
+                // Translators: Text that will be announced by the screen reader when no selection was selected.
+                let format = gettextrs::gettext("No suggestion selected");
+                obj.announce(&format, gtk::AccessibleAnnouncementPriority::Low);
+            }
+
             selection.set_selected(new_selected.unwrap_or(INVALID_LIST_POSITION));
             self.list_completions.scroll_to(
                 new_selected.unwrap_or_default(),
@@ -188,16 +205,16 @@ pub mod imp {
             Propagation::Stop
         }
 
-        fn try_fill_exact_match(&self, search: &str) {
-            self.obj().set_place(
-                self.completions
-                    .borrow()
-                    .into_iter()
-                    .flatten()
-                    .flat_map(|p| p.dynamic_cast::<Place>().ok())
-                    .find(|p| p.name() == Some(search.to_owned()))
-                    .as_ref(),
-            );
+        fn try_fill_exact_match(&self, search: &str) -> bool {
+            let exact = self
+                .completions
+                .borrow()
+                .into_iter()
+                .flatten()
+                .flat_map(|p| p.dynamic_cast::<Place>().ok())
+                .find(|p| p.name() == Some(search.to_owned()));
+            self.obj().set_place(exact.as_ref());
+            exact.is_some()
         }
 
         fn on_changed(&self) {
@@ -207,45 +224,74 @@ pub mod imp {
             let request_limiter = &self.request_limiter;
 
             let main_context = MainContext::default();
-            main_context.spawn_local(clone!(@strong obj,
-                                            @strong completions,
-                                            @strong request_limiter
-                                            => async move {
-                let entry = &obj;
-                let text = entry.text().to_string();
+            main_context.spawn_local(clone!(
+                #[strong]
+                obj,
+                #[strong]
+                completions,
+                #[strong]
+                request_limiter,
+                async move {
+                    let entry = &obj;
+                    let text = entry.text().to_string();
 
-                if text.len() < MIN_REQUEST_LEN {
-                    obj.set_place(None);
-                    return;
-                }
+                    if text.len() < MIN_REQUEST_LEN {
+                        obj.set_place(None);
+                        return;
+                    }
 
-                obj.imp().try_fill_exact_match(&text);
+                    // Try fill any exact match if available. If it could be filled, don't request.
+                    if obj.imp().try_fill_exact_match(&text) {
+                        return;
+                    }
 
-                let request = request_limiter.request(text).await;
+                    let request = request_limiter.request(text).await;
 
-                if let Some(request) = request {
-                    let places = obj.property::<HafasClient>("client").locations(LocationsOptions {query: request.clone(), ..Default::default()}).await;
+                    if let Some(request) = request {
+                        let places = obj
+                            .property::<Client>("client")
+                            .locations(LocationsOptions {
+                                query: request.clone(),
+                                ..Default::default()
+                            })
+                            .await;
 
-                    // XXX: Handle error case.
-                    if let Ok(places) = places {
-                        log::trace!("Got results back. Filling up completions.");
-                        let completions = completions.borrow();
-                        completions.splice(0, completions.n_items(), &places.into_iter().filter(|p| p.id().is_some()).collect::<Vec<_>>());
-                        drop(completions);
-                        obj.imp().try_fill_exact_match(&request);
+                        // XXX: Handle error case.
+                        if let Ok(places) = places {
+                            let places = places
+                                .into_iter()
+                                .filter(|p| p.id().is_some())
+                                .collect::<Vec<_>>();
+                            log::trace!("Got results back. Filling up completions.");
+                            let exact = places.iter().find(|p| p.name().as_ref() == Some(&request));
+
+                            let completions = completions.borrow();
+
+                            if exact.is_some() {
+                                obj.set_place(exact);
+                                completions.remove_all();
+                            } else {
+                                completions.splice(0, completions.n_items(), &places);
+                            }
+                            drop(completions);
+                        } else {
+                            log::error!("Error requesting places: {}", places.err().unwrap());
+                        }
+                    } else {
+                        log::trace!("No request needed");
                     }
-                } else {
-                    log::trace!("No request needed");
                 }
-            }));
+            ));
         }
 
         fn connect_changed(&self, obj: &super::StationEntry) {
-            self.obj()
-                .connect_changed(clone!(@strong obj => move |_entry| {
+            self.obj().connect_changed(clone!(
+                #[strong]
+                obj,
+                move |_entry| {
                     obj.imp().on_changed();
-                    obj.imp().selection.borrow().set_selected(INVALID_LIST_POSITION);
-                }));
+                }
+            ));
             self.on_changed();
         }
 
@@ -280,22 +326,37 @@ pub mod imp {
                 .set_model(Some(&selection.clone()));
 
             let factory = SignalListItemFactory::new();
-            factory.connect_setup(clone!(@weak obj => move |_, list_item| {
-                let place_item = PlaceListItem::new();
-                let list_item = list_item
-                    .downcast_ref::<ListItem>()
-                    .expect("The factory item to be a `ListItem`");
+            factory.connect_setup(clone!(
+                #[weak]
+                obj,
+                move |_, list_item| {
+                    let place_item = PlaceListItem::new();
+                    let list_item = list_item
+                        .downcast_ref::<ListItem>()
+                        .expect("The factory item to be a `ListItem`");
 
-                list_item.set_child(Some(&place_item));
-                list_item
-                    .property_expression("item")
-                    .bind(&place_item, "place", Widget::NONE);
+                    list_item.set_child(Some(&place_item));
+                    list_item
+                        .property_expression("item")
+                        .bind(&place_item, "place", Widget::NONE);
 
-                place_item.connect_local("pressed", false, clone!(@weak obj, @weak place_item => @default-return None, move |_| {
-                    obj.set_place(place_item.place().as_ref());
-                    None
-                }));
-            }));
+                    place_item.connect_local(
+                        "pressed",
+                        false,
+                        clone!(
+                            #[weak]
+                            obj,
+                            #[weak]
+                            place_item,
+                            #[upgrade_or_default]
+                            move |_| {
+                                obj.set_place(place_item.place().as_ref());
+                                None
+                            }
+                        ),
+                    );
+                }
+            ));
             self.list_completions.set_factory(Some(&factory));
         }
 
@@ -308,7 +369,7 @@ pub mod imp {
                     c.set_visible(false);
                     c.unparent();
                     break;
-                } else if c.has_css_class("header") {
+                } else if c.has_css_class("header") || c.has_css_class("editable-area") {
                     child = c.first_child();
                 } else {
                     child = c.next_sibling();
@@ -346,19 +407,39 @@ pub mod imp {
 
             self.completions.borrow().connect_notify_local(
                 Some("n-items"),
-                clone!(@strong obj => move |_, _| {
-                    obj.imp().update_popover_visible();
-                }),
+                clone!(
+                    #[strong]
+                    obj,
+                    move |list, _| {
+                        obj.imp().update_popover_visible();
+
+                        let n = list.n_items();
+                        if n > 0 {
+                            // Translators: Text that will be announced by the screen reader when suggestions changed in the station entry.
+                            let format = gettextrs::ngettext(
+                                "one suggestion, not selected",
+                                "{n} suggestions, none selected",
+                                n,
+                            )
+                            .replace("{n}", &n.to_string());
+                            obj.announce(&format, gtk::AccessibleAnnouncementPriority::Low);
+                        }
+                    }
+                ),
             );
 
-            self.obj()
-                .connect_css_classes_notify(clone!(@strong obj => move |_| {
+            self.obj().connect_css_classes_notify(clone!(
+                #[strong]
+                obj,
+                move |_| {
                     // Update when e.g. focus changed.
                     obj.imp().update_popover_visible();
-                }));
+                }
+            ));
 
             obj.connect_notify_local(Some("place"), |obj, _| {
                 obj.notify("set");
+                obj.imp().update_popover_visible();
             });
         }
 
@@ -368,7 +449,7 @@ pub mod imp {
                 v.extend(vec![
                     ParamSpecObject::builder::<Place>("place").build(),
                     ParamSpecBoolean::builder("set").read_only().build(),
-                    ParamSpecObject::builder::<HafasClient>("client").build(),
+                    ParamSpecObject::builder::<Client>("client").build(),
                     ParamSpecObject::builder::<ListStore>("completions")
                         .read_only()
                         .build(),
@@ -389,13 +470,13 @@ pub mod imp {
                     );
                 }
                 "client" => {
-                    let obj = value.get::<Option<HafasClient>>().expect(
-                        "Property `client` of `StationEntry` has to be of type `HafasClient`",
-                    );
+                    let obj = value
+                        .get::<Option<Client>>()
+                        .expect("Property `client` of `StationEntry` has to be of type `Client`");
 
                     if let Some(obj) = &obj {
                         let s = self.obj();
-                        obj.connect_local("provider-changed", true, clone!(@weak s => @default-return None, move |_| {
+                        obj.connect_local("provider-changed", true, clone!(#[weak] s, #[upgrade_or_default] move |_| {
                             log::trace!("Station-entry got provider change from hafas_client. Resetting");
                             s.set_place(None);
                             s.imp().on_changed();
